class CorpusAnalysisApp {
  constructor(corpusId) {
    this.data = {};

    // HTML elements
    this.elements = {
      container: document.querySelector('#corpus-analysis-app-container'),
      extensionTabs: document.querySelector('#corpus-analysis-app-extension-tabs'),
      initModal: document.querySelector('#corpus-analysis-app-init-modal'),
      overview: document.querySelector('#corpus-analysis-app-overview')
    };
    // Materialize elements
    this.elements.m = {
      extensionTabs: M.Tabs.init(this.elements.extensionTabs),
      initModal: M.Modal.init(this.elements.initModal, {dismissible: false})
    };

    this.extensions = {};

    this.settings = {
      corpusId: corpusId
    };
  }

  init() {
    this.disableActionElements();
    this.elements.m.initModal.open();
    // Init data
    this.data.cQiClient = new CQiClient(this.settings.corpusId);
    this.data.cQiClient.connect()
      .then(cQiStatus => {
        return this.data.cQiClient.corpora.get(`NOPAQUE_${this.settings.corpusId}`);
      })
      .then(
        cQiCorpus => {
          this.data.corpus = {o: cQiCorpus};
          this.data.corpus.o.getVisualizationData()
            .then(
              (data) => {
                console.log(data);
                this.renderGeneralCorpusInfo(data);
                this.renderTextInfoList(data);
                this.renderTextProportionsGraphic(data);
                this.renderFrequenciesGraphic(data);
                this.renderBoundsGraphic(data);
              }
            );
          // this.data.corpus.o.getCorpusData()
          //   .then(corpusData => {
          //     console.log(corpusData);
          //     this.renderGeneralCorpusInfo(corpusData);
          //     this.renderTextInfoList(corpusData);
          //     this.renderTextProportionsGraphic(corpusData);
          //     this.renderFrequenciesGraphic(corpusData);
          //     this.renderBoundsGraphic(corpusData);
          //   });
          // TODO: Don't do this hgere
          cQiCorpus.updateDb();
          this.enableActionElements();
          for (let extension of Object.values(this.extensions)) {extension.init();}
          this.elements.m.initModal.close();
        },
        cQiError => {
          let errorsElement = this.elements.initModal.querySelector('.errors');
          let progressElement = this.elements.initModal.querySelector('.progress');
          errorsElement.innerText = JSON.stringify(cQiError);
          errorsElement.classList.remove('hide');
          progressElement.classList.add('hide');
          if ('payload' in cQiError && 'code' in cQiError.payload && 'msg' in cQiError.payload) {
            app.flash(`${cQiError.payload.code}: ${cQiError.payload.msg}`, 'error');
          }
        }
      );

  
    // Add event listeners
    for (let extensionSelectorElement of this.elements.overview.querySelectorAll('.extension-selector')) {
      extensionSelectorElement.addEventListener('click', () => {
        this.elements.m.extensionTabs.select(extensionSelectorElement.dataset.target);
      });
    }
  }

  registerExtension(extension) {
    if (extension.name in this.extensions) {
      console.error(`Can't register extension ${extension.name}: Already registered`);
      return;
    }
    this.extensions[extension.name] = extension;
    if ('cQiClient' in this.data && this.data.cQiClient.connected) {extension.init();}
  }

  disableActionElements() {
    let actionElements = this.elements.container.querySelectorAll('.corpus-analysis-action');
    for (let actionElement of actionElements) {
      if (actionElement.nodeName === 'INPUT') {
        actionElement.disabled = true;
      } else if (actionElement.nodeName === 'SELECT') {
        actionElement.parentNode.querySelector('input.select-dropdown').disabled = true;
      } else {
        actionElement.classList.add('disabled');
      }
    }
  }

  enableActionElements() {
    let actionElements = this.elements.container.querySelectorAll('.corpus-analysis-action');
    for (let actionElement of actionElements) {
      if (actionElement.nodeName === 'INPUT') {
        actionElement.disabled = false;
      } else if (actionElement.nodeName === 'SELECT') {
        actionElement.parentNode.querySelector('input.select-dropdown').disabled = false;
      } else {
        actionElement.classList.remove('disabled');
      }
    }
  }

  renderGeneralCorpusInfo(corpusData) {
    document.querySelector('.corpus-num-tokens').innerHTML = corpusData.corpus.counts.token;
    document.querySelector('.corpus-num-s').innerHTML = corpusData.corpus.counts.s;
    document.querySelector('.corpus-num-unique-words').innerHTML = Object.entries(corpusData.corpus.freqs.word).length;
    document.querySelector('.corpus-num-unique-lemmas').innerHTML = Object.entries(corpusData.corpus.freqs.lemma).length;
    document.querySelector('.corpus-num-unique-pos').innerHTML = Object.entries(corpusData.corpus.freqs.pos).length;
    document.querySelector('.corpus-num-unique-simple-pos').innerHTML = Object.entries(corpusData.corpus.freqs.simple_pos).length;
  }

  renderTextInfoList(corpusData) {
    let corpusTextInfoListElement = document.querySelector('.corpus-text-info-list');
    let corpusTextInfoList = new CorpusTextInfoList(corpusTextInfoListElement);
    let texts = corpusData.s_attrs.text.lexicon;
    let textData = [];
    for (let i = 0; i < Object.entries(texts).length; i++) {
      let resource = {
        title: corpusData.values.s_attrs.text[i].title,
        publishing_year: corpusData.values.s_attrs.text[i].publishing_year,
        num_tokens: corpusData.s_attrs.text.lexicon[i].counts.token,
        num_sentences: corpusData.s_attrs.text.lexicon[i].counts.s,
        num_unique_words: Object.entries(corpusData.s_attrs.text.lexicon[i].freqs.word).length,
        num_unique_lemmas: Object.entries(corpusData.s_attrs.text.lexicon[i].freqs.lemma).length,
        num_unique_pos: Object.entries(corpusData.s_attrs.text.lexicon[i].freqs.pos).length,
        num_unique_simple_pos: Object.entries(corpusData.s_attrs.text.lexicon[i].freqs.simple_pos).length
      };
  
      textData.push(resource);
    }

    corpusTextInfoList.add(textData);

    let textCountChipElement = document.querySelector('.text-count-chip');
    textCountChipElement.innerHTML = `Text count: ${corpusData.corpus.counts.text}`;
  }

  renderTextProportionsGraphic(corpusData) {
    let textProportionsGraphicElement = document.querySelector('#text-proportions-graphic');
    let texts = Object.entries(corpusData.s_attrs.text.lexicon);
    let graphData = [
      {
        values: texts.map(text => text[1].counts.token),
        labels: texts.map(text => `${corpusData.values.s_attrs.text[text[0]].title} (${corpusData.values.s_attrs.text[text[0]].publishing_year})`),
        type: 'pie'
      }
    ];
    let config = {responsive: true};

    Plotly.newPlot(textProportionsGraphicElement, graphData, config);
  }

  renderFrequenciesGraphic(corpusData) {
    let frequenciesTokenCategoryDropdownElement = document.querySelector('[data-target="frequencies-token-category-dropdown"]');
    let frequenciesTokenCategoryDropdownListElement = document.querySelector("#frequencies-token-category-dropdown");
    let frequenciesGraphicElement = document.querySelector('#frequencies-graphic');
    let texts = Object.entries(corpusData.s_attrs.text.lexicon);
    
    
    frequenciesTokenCategoryDropdownListElement.addEventListener('click', (event) => {
      frequenciesTokenCategoryDropdownElement.firstChild.textContent = event.target.innerHTML;
      this.renderFrequenciesGraphic(corpusData);
    });
    
    let tokenCategory = frequenciesTokenCategoryDropdownElement.firstChild.textContent.toLowerCase();
    
    let graphData = this.createFrequenciesGraphData(tokenCategory, texts, corpusData);
    let graphLayout = {
      barmode: 'stack',
      type: 'bar'
    };
    let config = {responsive: true};
    
    Plotly.newPlot(frequenciesGraphicElement, graphData, graphLayout, config);
  }

  createFrequenciesGraphData(category, texts, corpusData) {
    let graphData = [];
    let sortedData = Object.entries(corpusData.corpus.freqs[category]).sort((a, b) => b[1] - a[1]).slice(0, 5);
  
    for (let item of sortedData) {
      let data = {
        x: texts.map(text => `${corpusData.values.s_attrs.text[text[0]].title} (${corpusData.values.s_attrs.text[text[0]].publishing_year})`),
        y: texts.map(text => text[1].freqs[category][item[0]]),
        name: corpusData.values.p_attrs[category][item[0]],
        type: 'bar'
      };
      graphData.push(data);
    }
  
    return graphData;
  }

  renderBoundsGraphic(corpusData) {
      let boundsGraphicElement = document.querySelector('#bounds-graphic');

      let graphData = [];
      let texts = Object.entries(corpusData.s_attrs.text.lexicon);

      graphData = [{
        type: 'bar',
        x: texts.map(text => text[1].bounds[1] - text[1].bounds[0]),
        y: texts.map(text => corpusData.values.s_attrs.text[text[0]].title),
        base: texts.map(text => text[1].bounds[0]),
        text: texts.map(text => `${corpusData.values.s_attrs.text[text[0]].title} (${corpusData.values.s_attrs.text[text[0]].publishing_year})`),
        orientation: 'h',
        hovertemplate: '%{base} - %{x} <br>%{y}',
        showlegend: false
      }];

      let graphLayout = {
        barmode: 'stack',
        type: 'bar',
        showgrid: false,
        xaxis: {
          rangemode: 'nonnegative',
          autorange: true
        },
        yaxis: {
          autorange: true,
          showticklabels: false
        }
      };

      let config = {responsive: true};
    
      Plotly.newPlot(boundsGraphicElement, graphData, graphLayout, config);
  }
}
